from constants import *

class ChangeHeader:
    """This plugin changes header.
    
        It takes the following arguments: 
        Action:            enum   (Replace/Delete)
        Header Name:       string (Name of the Header)
        Header Value:      string (New Value of the Header)
        Request/Response:  enum   (Request/Response/Both)
        
        We can create such a rule using this plugin 
        example:
            User-Agent Changer 
            We can change our User-Agent value Internet Explorer to IronCurtain by this way
            
            Args Action: Replace HeaderName: User-Agent HeaderValue: IronCurtain/0.0.1 Request/Response: Request
    """
    Args = [("Action", "Action", "enum", ("Replace", "Delete")), 
             ("HeaderName", "Header Name", "string"),
             ("HeaderValue", "Header Value", "string"),
             ("ReqRes", "Request/Response", "enum", ("Request", "Response", "Both"))] 
    
    Desc = "Changes request or respose headers"

    ApplyTime = BEFORE_BODY
    
    def __init__(self, params):
        self.params = params
    
    def act_on_req(self, cReq):
        if self.params["ReqRes"] != "Response":
            self.modify_headers(cReq)
        return ACTION_FORWARD, ""
    
    def act_on_res(self, cRes):
        if self.params["ReqRes"] != "Request":
            self.modify_headers(cRes)
        return ACTION_FORWARD, ""
            
    def modify_headers(self, cReqRes):
        if self.params["Action"] == "Delete":
            cReqRes.delete_header(self.params["HeaderName"], self.params["HeaderValue"])
        elif self.params["Action"] == "Replace":
            cReqRes.replace_header(self.params["HeaderName"], self.params["HeaderValue"])
