"""Filter images by size"""

import Image
import os
import re
import cStringIO as StringIO
from constants import *

class ImageSize:
    """This plugin blocks images with given size interval. """
    
    Args = [
            ("ImgWidth", "Image Width", "string"),
            ("ImgHeight", "Image Height", "string"),
            ("ImgProp", "Image Property","enum",("Equal","Larger","Smaller")),
            ("ImgFormat", "Image Format", "string"),                      
           ]    
    
    Desc = "Blocks images with the specified size,format"
    
    ApplyTime = AFTER_4KB
    
    def __init__ (self, params):
        
        # minimal amount of image data for PIL to read header info:
        # 4096 bytes is enough for most images; the value is increased
        # when it is not big enough
        self.min_bufsize = 4096
        
	try: 
            parsedImgWidth = int(params["ImgWidth"])
            if (parsedImgWidth <= 0):
                raise ParamError, PARAM_CONTENT_ERROR + "ImgWidth" + " should be positive"
        except ValueError: 
            raise ParamError, PARAM_TYPE_MISMATCH + "ImgWidth" + " should be integer"
        
        try: 
            parsedImgHeigth = int(params["ImgHeight"])
            if (parsedImgHeight <= 0):
                raise ParamError, PARAM_CONTENT_ERROR + "ImgHeight" + " should be positive"
        except ValueError: 
            raise ParamError, PARAM_TYPE_MISMATCH + "ImgHeight" + " should be integer"
    
        print "\n\n\n\n\nexception atmadim\n\n\n\n\n\n\n"
        if params["ImgFormat"]:
            self.imgsize_sizes = [((int(params["ImgWidth"]), int(params["ImgHeight"])),[params["ImgFormat"]])]
        else:
           self.imgsize_sizes = [((int(params["ImgWidth"]), int(params["ImgHeight"])),[])] 
        self.params = params
        
    def act_on_req(self, cReq):
        return ACTION_FORWARD
    
    def act_on_res(self, cRes):
    
        if not cRes.get_header("content-type") or not re.match("image\/*", cRes.get_header("content-type")):
            return ACTION_FORWARD
        
        self.image_filter(cRes, imgsize_blocked=False, imgsize_sizes=self.imgsize_sizes, url="")
        return ACTION_FORWARD
        
    def image_filter (self, cRes, **attrs):
        # put an image instead 
        data = cRes.body
        if not data:
            # do not block this image
            return
        if attrs['imgsize_blocked']:
            # block this image
            print "attrs[imgsize_blocked]"
            return
        buf = StringIO.StringIO()
        if buf.closed:
            # do not block this image
            return
        buf.write(data)
        attrs['imgsize_blocked'] = \
              self.check_sizes(buf, attrs['imgsize_sizes'], attrs['url'])
        buf.close()
        if attrs['imgsize_blocked']:
            blockdata = StringIO.StringIO()
            im = Image.open(os.path.join("..", "data", "images", "transparent.gif"))
            im.thumbnail((self.img.size[0], self.img.size[1]))
            im.save(blockdata, "GIF")
            blockdata.seek(0)
            _data = blockdata.read()
            blockdata.close()
            
            cRes.replace_header("content-type", "image/gif")   
            cRes.replace_header("content-size", len(_data))
            cRes.body = _data
            return ACTION_ANSWER
            #print "Replaced!!!!!!!!!!!!!!!"
        return ACTION_FORWARD

    def check_sizes (self, buf, sizes, url, finish=False):
        pos = buf.tell()
        assert pos > 0
        try:
            buf.seek(0)
            self.img = Image.open(buf, 'r')
            buf.seek(pos)
            #print "img.size", self.img.size, "img.format", self.img.format
            
            for size, formats in sizes:
                if self.img.size[0] * self.img.size[1] < 101:
                    return True
                if (self.params["ImgProp"] == "Equal" and size == self.img.size) or \
                (self.params["ImgProp"] == "Larger" and size[0]*size[1] <= self.img.size[0]*self.img.size[1]) or \
                (self.params["ImgProp"] == "Smaller" and size[0]*size[1] >= self.img.size[0]*self.img.size[1]):
                    # size matches, look for format restriction
                    if not formats:
                        # blocking image size
                        return True
                    if self.img.format.upper() in formats or self.img.format.lower() in formats:
                        return True
        except IOError:
            if finish:
                pass
	    else:
                assert pos > self.min_bufsize
                buf.seek(pos)
                self.min_bufsize = pos+4096
                assert buf.tell() < self.min_bufsize
        return False
